@@ -8,6 +8,10 @@
  * Please see the LICENSE.TXT file for specific restrictions.
  */
 
+/* Shared headers */
+#include "pginternal.h"
+#include "pgmodule.h"
+
 /* Deallocate source object. */
 static void
 source_dealloc(sourceObject *self)
@@ -66,6 +70,8 @@ static PyObject *
 source_getattr(sourceObject *self, PyObject *nameobj)
 {
     const char *name = PyUnicode_AsUTF8(nameobj);
+    if (!name)
+        return NULL;
 
     /* pg connection object */
     if (!strcmp(name, "pgcnx")) {
@@ -796,7 +802,7 @@ static PyMethodDef source_methods[] = {
 static char source__doc__[] = "PyGreSQL source object";
 
 /* Source type definition */
-static PyTypeObject sourceType = {
+PyTypeObject sourceType = {
     PyVarObject_HEAD_INIT(NULL, 0) "pgdb.Source", /* tp_name */
     sizeof(sourceObject),                         /* tp_basicsize */
     0,                                            /* tp_itemsize */
